<?php

namespace KnitPay\Extensions\MicroJobEngine;

use Pronamic\WordPress\Pay\AbstractPluginIntegration;
use Pronamic\WordPress\Pay\Plugin;
use Pronamic\WordPress\Pay\Payments\Payment;
use Pronamic\WordPress\Pay\Payments\PaymentStatus as Core_Statuses;
use ET_Order;
use MJE_Order;

/**
 * Title: Micro Job Engine extension
 * Description:
 * Copyright: 2020-2021 Knit Pay
 * Company: Knit Pay
 *
 * @author  knitpay
 * @since   4.7.0
 */
class Extension extends AbstractPluginIntegration {
	/**
	 * Slug
	 *
	 * @var string
	 */
	const SLUG = 'microjobengine';

	/**
	 * Constructs and initialize Micro Job Engine extension.
	 */
	public function __construct() {
		parent::__construct(
			array(
				'name' => __( 'Micro Job Engine', 'knit-pay-lang' ),
			)
		);

		// Dependencies.
		$dependencies = $this->get_dependencies();

		$dependencies->add( new MicroJobEngineDependency() );
	}

	/**
	 * Setup plugin integration.
	 *
	 * @return void
	 */
	public function setup() {
		add_filter( 'pronamic_payment_source_text_' . self::SLUG, array( $this, 'source_text' ), 10, 2 );
		add_filter( 'pronamic_payment_source_description_' . self::SLUG, array( $this, 'source_description' ), 10, 2 );
		add_filter( 'pronamic_payment_source_url_' . self::SLUG, array( $this, 'source_url' ), 10, 2 );

		// Check if dependencies are met and integration is active.
		if ( ! $this->is_active() ) {
			return;
		}

		add_filter( 'pronamic_payment_redirect_url_' . self::SLUG, array( $this, 'redirect_url' ), 10, 2 );
		add_action( 'pronamic_payment_status_update_' . self::SLUG, array( $this, 'status_update' ), 10 );

		add_action( 'after_setup_theme', array( $this, 'require_plugin_files' ) );
		add_filter( 'mje_payment_gateway_setting_sections', array( $this, 'add_gateway_setting_fields' ) );
		add_filter( 'et_build_payment_visitor', array( $this, 'build_payment_visitor' ), 10, 3 );
	}

	public function require_plugin_files() {
		require_once 'Gateway.php';
		require_once 'Helper.php';
		require_once 'KnitPayVisitor.php';
		new Gateway();
	}

	/**
	 * Add payment visitor for STRIPE
	 *
	 * @param object   $class
	 * @param string   $payment_type
	 * @param ET_Order $order
	 * @return object $class
	 */
	public function build_payment_visitor( $class, $payment_type, $order ) {
		if ( $payment_type == 'KNIT_PAY' ) {
			$class = new KnitPayVisitor( $order );
		}

		return $class;
	}

	public static function add_gateway_setting_fields( $sections ) {

		$sections['knit-pay'] = array(
			'args'   => array(
				'title' => __( 'Knit Pay', 'knit-pay-lang' ),
				'id'    => 'knit-pay',
				'class' => '',
				'icon'  => '',
			),
			'groups' => array(
				array(
					'args'   => array(
						'title' => __( 'Knit Pay', 'knit-pay-lang' ),
						'id'    => '',
						'class' => '',
						'desc'  => '',
					),
					'fields' => array(
						array(
							'id'       => 'knit_pay',
							'class'    => '',
							'type'     => 'combine',
							'title'    => __( 'Knit Pay Settings', 'knit-pay-lang' ),
							'desc'     => '',
							'name'     => 'knit_pay',
							'children' => array(
								array(
									'id'    => 'enable',
									'class' => '',
									'type'  => 'switch',
									'title' => __( 'Enable Knit Pay', 'knit-pay-lang' ),
									'desc'  => '',
									'name'  => 'enable',
								),
								array(
									'id'    => 'title',
									'class' => '',
									'type'  => 'text',
									'title' => __( 'Title', 'knit-pay-lang' ),
									'desc'  => __( 'It will be displayed on checkout page.', 'knit-pay-lang' ),
									'name'  => 'title',
								),
								array(
									'id'    => 'config_id',
									'class' => '',
									'type'  => 'select',
									'title' => __( 'Configuration', 'knit-pay-lang' ),
									'desc'  => __( 'Configurations can be created in Knit Pay gateway configurations page at <a href="' . admin_url() . 'edit.php?post_type=pronamic_gateway">"Knit Pay >> Configurations"</a>.', 'knit-pay-lang' ),
									'name'  => 'config_id',
									'data'  => Plugin::get_config_select_options(),
								),
								array(
									'id'    => 'payment_description',
									'class' => '',
									'type'  => 'text',
									'title' => __( 'Payment Description', 'knit-pay-lang' ),
									'desc'  => sprintf( __( 'Available tags: %s', 'knit-pay-lang' ), sprintf( '<code>%s</code>', '{order_id}' ) ),
									'name'  => 'payment_description',
								),
								array(
									'id'    => 'icon_url',
									'class' => '',
									'type'  => 'text',
									'title' => __( 'Icon URL', 'knit-pay-lang' ),
									'desc'  => __( 'This controls the icon which the user sees during checkout. Keep it blank to use default Icon.', 'knit-pay-lang' ) . '<br>' . __( 'Recommended Size 78x75', 'knit-pay-lang' ),
									'name'  => 'icon_url',
								),
							),
						),
					),
				),
			),
		);

		return $sections;
	}

	/**
	 * Payment redirect URL filter.
	 *
	 * @param string  $url     Redirect URL.
	 * @param Payment $payment Payment.
	 *
	 * @return string
	 */
	public static function redirect_url( $url, $payment ) {
		switch ( $payment->get_status() ) {
			case Core_Statuses::CANCELLED:
			case Core_Statuses::EXPIRED:
			case Core_Statuses::FAILURE:
				$url = et_get_page_link( 'cancel-payment' );

				break;

			case Core_Statuses::SUCCESS:
				$url = $payment->get_meta( 'success_return_url' );
				break;

			case Core_Statuses::RESERVED:
			case Core_Statuses::OPEN:
			default:
		}

		return $url;
	}

	/**
	 * Update the status of the specified payment
	 *
	 * @param Payment $payment Payment.
	 */
	public static function status_update( Payment $payment ) {
		$source_id  = (int) $payment->get_source_id();
		$order      = new MJE_Order( $source_id );
		$order_data = $order->get_order_data();

		$order->set_payment_code( $payment->get_transaction_id() );

		switch ( $payment->get_status() ) {
			case Core_Statuses::CANCELLED:
			case Core_Statuses::EXPIRED:
			case Core_Statuses::FAILURE:
				$order->set_status( 'draft' );
				$post_status = 'draft';

				break;
			case Core_Statuses::SUCCESS:
				$order->set_status( 'publish' );
				$post_status = 'publish';

				break;
			case Core_Statuses::OPEN:
			default:
				$order->set_status( 'pending' );
				$post_status = 'pending';

				break;
		}
		$order->update_order();
		wp_update_post(
			array(
				'ID'          => $order_data['product_id'],
				'post_status' => $post_status,
				'post_title'  => reset( $order_data['products'] )['NAME'],

			)
		);
	}

	/**
	 * Source column
	 *
	 * @param string  $text    Source text.
	 * @param Payment $payment Payment.
	 *
	 * @return string $text
	 */
	public function source_text( $text, Payment $payment ) {
		$mjob_order_id = $this->get_mjob_order_id( $payment );
		$text          = __( 'Micro Job Engine', 'knit-pay-lang' ) . '<br />';

		$text .= sprintf(
			'<a href="%s">%s</a>',
			get_edit_post_link( $mjob_order_id ),
			/* translators: %s: source id */
			sprintf( __( 'Order %s', 'knit-pay-lang' ), $mjob_order_id )
		);

		return $text;
	}

	/**
	 * Source description.
	 *
	 * @param string  $description Description.
	 * @param Payment $payment     Payment.
	 *
	 * @return string
	 */
	public function source_description( $description, Payment $payment ) {
		return __( 'Micro Job Engine Order', 'knit-pay-lang' );
	}

	/**
	 * Source URL.
	 *
	 * @param string  $url     URL.
	 * @param Payment $payment Payment.
	 *
	 * @return string
	 */
	public function source_url( $url, Payment $payment ) {
		return get_edit_post_link( $this->get_mjob_order_id( $payment ) );
	}

	private function get_mjob_order_id( Payment $payment ) {
		$source_id  = (int) $payment->get_source_id();
		$order      = new MJE_Order( $source_id );
		$order_data = $order->get_order_data();
		return $order_data['product_id'];
	}

}
